@@ -71,7 +71,7 @@
 
 // program information
 #define INFO_PROGRAM "FATSort Utility"
-#define INFO_VERSION "0.9.7"
+#define INFO_VERSION "0.9.7.1"
 #define INFO_AUTHOR "Boris Leidner <fatsort(at)formenos.de>"
 #define INFO_HEADER INFO_PROGRAM " " INFO_VERSION " by " INFO_AUTHOR
 #define INFO_USAGE "\n"	"Usage: fatsort [options] device\n" \
@@ -298,7 +298,7 @@ struct sDirEntryList *
 	assert(sname != NULL);
 	assert(lname != NULL);
 	assert(sde != NULL);
-
+	
 	struct sDirEntryList *tmp;
 
 	if ((tmp=malloc(sizeof(struct sDirEntryList)))==NULL) {
@@ -558,7 +558,7 @@ int32_t getFATEntry(FILE *fd, struct sBootSector *bs, u_int32_t cluster, u_int32
 	assert(bs != NULL);
 	assert(data != NULL);
 
-	u_int32_t FATOffset, FATSz, BSOffset;
+	off_t FATOffset, FATSz, BSOffset;
 	int32_t FATType;
 
 	*data=0;
@@ -572,9 +572,9 @@ int32_t getFATEntry(FILE *fd, struct sBootSector *bs, u_int32_t cluster, u_int32
 	FATType = getFATType(bs);
 
 	if (FATType == 16) {
-		FATOffset = cluster * 2;
-		BSOffset = SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
-		if (fseek(fd, BSOffset, SEEK_SET) == -1) {
+		FATOffset = (off_t)cluster * 2;
+		BSOffset = (off_t)SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
+		if (fseeko(fd, BSOffset, SEEK_SET) == -1) {
 			myerror("Seek error!");
 			return -1;
 		}
@@ -584,9 +584,9 @@ int32_t getFATEntry(FILE *fd, struct sBootSector *bs, u_int32_t cluster, u_int32
 		}
 		*data=SwapInt32(*data);
 	} else if (FATType == 32) {
-		FATOffset = cluster * 4;
-		BSOffset = SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
-		if (fseek(fd, BSOffset, SEEK_SET) == -1) {
+		FATOffset = (off_t)cluster * 4;
+		BSOffset = (off_t)SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
+		if (fseeko(fd, BSOffset, SEEK_SET) == -1) {
 			myerror("Seek error!");
 			return -1;
 		}
@@ -616,7 +616,7 @@ int32_t putFATEntry(FILE *fd, struct sBootSector *bs, u_int32_t cluster, u_int32
 	assert(fd != NULL);
 	assert(bs != NULL);
 
-	u_int32_t FATOffset, FATSz, BSOffset;
+	off_t FATOffset, FATSz, BSOffset;
 	u_int32_t value, i;
 	int32_t FATType;
 
@@ -629,24 +629,24 @@ int32_t putFATEntry(FILE *fd, struct sBootSector *bs, u_int32_t cluster, u_int32
 	FATType = getFATType(bs);
 
 	if (FATType == 16) {
-		FATOffset = cluster * 2;
-		BSOffset = SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
-		fseek(fd, BSOffset, SEEK_SET);
+		FATOffset = (off_t)cluster * 2;
+		BSOffset = (off_t)SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
+		fseeko(fd, BSOffset, SEEK_SET);
 		data=SwapInt32(data);
 		if (fwrite(&data, 2, 1, fd)<1) {
 			myerror("Failed to write to file!");
 			return -1;
 		}
 	} else if (FATType == 32) {
-		FATOffset = cluster * 4;
-		BSOffset = SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
+		FATOffset = (off_t)cluster * 4;
+		BSOffset = (off_t)SwapInt16(bs->BS_RsvdSecCnt) * SwapInt16(bs->BS_BytesPerSec) + FATOffset;
 		if (getFATEntry(fd, bs, cluster, &value)==-1) {
 			myerror("Failed to get FAT entry!");
 			return -1;
 		}
 		value = (value & 0xf0000000) | (data & 0x0fffffff);
 		for(i=0; i<bs->BS_NumFATs; i++) {
-			fseek(fd, BSOffset+i*FATSz*bs->BS_BytesPerSec, SEEK_SET);
+			fseeko(fd, BSOffset+i*FATSz*bs->BS_BytesPerSec, SEEK_SET);
 			value=SwapInt32(value);
 			if (fwrite(&value, 4, 1, fd) < 1) {
 				myerror("Failed to write to file!");
@@ -733,7 +733,7 @@ int32_t parseEntry(FILE *fd, union sDirEntry *de) {
 	return 1; // short dir entry
 }
 
-u_int32_t getClusterOffset(struct sBootSector *bs, u_int32_t cluster) {
+off_t getClusterOffset(struct sBootSector *bs, u_int32_t cluster) {
 /*
 	returns the offset of a specific cluster in the
 	data region of the file system
@@ -753,7 +753,7 @@ u_int32_t getClusterOffset(struct sBootSector *bs, u_int32_t cluster) {
 	RootDirSectors = ((SwapInt16(bs->BS_RootEntCnt) * DIR_ENTRY_SIZE) + (SwapInt16(bs->BS_BytesPerSec) - 1)) / SwapInt16(bs->BS_BytesPerSec);
 	FirstDataSector = (SwapInt16(bs->BS_RsvdSecCnt) + (bs->BS_NumFATs * FATSz) + RootDirSectors);
 
-	return (((cluster - 2) * bs->BS_SecPerClus) + FirstDataSector) * SwapInt16(bs->BS_BytesPerSec);
+	return (((off_t)(cluster - 2) * bs->BS_SecPerClus) + FirstDataSector) * SwapInt16(bs->BS_BytesPerSec);
 
 }
 
@@ -780,7 +780,7 @@ int32_t parseClusterChain(FILE *fd, struct sBootSector *bs, struct sClusterChain
 	llist = NULL;
 	lname[0]='\0';
 	while (chain != NULL) {
-		fseek(fd, getClusterOffset(bs, chain->cluster), SEEK_SET);
+		fseeko(fd, getClusterOffset(bs, chain->cluster), SEEK_SET);
 		for (j=1;j<=maxEntries;j++) {
 			entries++;
 			ret=parseEntry(fd, &de);
@@ -1029,7 +1029,7 @@ int32_t writeClusterChain(FILE *fd, struct sBootSector *bs, struct sDirEntryList
 	chain=chain->next;	// we don't need to look at the head element
 
 	MaxEntries = bs->BS_SecPerClus * SwapInt16(bs->BS_BytesPerSec) / DIR_ENTRY_SIZE;
-	if (fseek(fd, getClusterOffset(bs, chain->cluster), SEEK_SET)==-1) {
+	if (fseeko(fd, getClusterOffset(bs, chain->cluster), SEEK_SET)==-1) {
 		myerror("Seek error!");
 		return -1;
 	}
@@ -1059,7 +1059,7 @@ int32_t writeClusterChain(FILE *fd, struct sBootSector *bs, struct sDirEntryList
 				tmp=tmp->next;
 			}
 			chain=chain->next; entries=p->entries - (MaxEntries - entries);	// next cluster
-			if (fseek(fd, getClusterOffset(bs, chain->cluster), SEEK_SET)==-1) {
+			if (fseeko(fd, getClusterOffset(bs, chain->cluster), SEEK_SET)==-1) {
 				myerror("Seek error!");
 				return -1;
 			}
@@ -1186,7 +1186,8 @@ int32_t sort_FAT16_rootdir(FILE *fd, struct sBootSector *bs) {
 	assert(fd != NULL);
 	assert(bs != NULL);
 
-	u_int32_t FATSz, BSOffset, i, ret;
+	off_t BSOffset;
+	u_int32_t FATSz, i, ret;
 	u_int32_t entries,count=0, c, value;
 	char newpath[MAX_PATH_LEN+1]={0};
 	char empty[32]={0};
@@ -1200,9 +1201,9 @@ int32_t sort_FAT16_rootdir(FILE *fd, struct sBootSector *bs) {
 		FATSz = SwapInt32(bs->FATxx.FAT32.BS_FATSz32);
 	}
 
-	BSOffset = (SwapInt16(bs->BS_RsvdSecCnt) + bs->BS_NumFATs * FATSz)* SwapInt16(bs->BS_BytesPerSec);
+	BSOffset = ((off_t)SwapInt16(bs->BS_RsvdSecCnt) + bs->BS_NumFATs * FATSz)* SwapInt16(bs->BS_BytesPerSec);
 
-	fseek(fd, BSOffset, SEEK_SET);
+	fseeko(fd, BSOffset, SEEK_SET);
 
 	if (OPT_LIST) {
 		printf("/\n");
@@ -1225,7 +1226,7 @@ int32_t sort_FAT16_rootdir(FILE *fd, struct sBootSector *bs) {
 	if (!OPT_LIST) {
 		infomsg("Sorting directory /\n");
 
-		fseek(fd, BSOffset, SEEK_SET);
+		fseeko(fd, BSOffset, SEEK_SET);
 
 		// write the sorted entries back to the fs
 		if (writeList(fd, &list) == -1) {
@@ -1334,15 +1335,15 @@ int32_t printFSInfo(char *filename) {
 	printf("FAT size:\t\t\t%u sectors (%u bytes)\n", FATSz, FATSz * SwapInt16(bs.BS_BytesPerSec));
 	printf("Cluster size:\t\t\t%u bytes\n", bs.BS_SecPerClus * SwapInt16(bs.BS_BytesPerSec));
 	printf("Cluster count:\t\t\t%u\n", cluster);
-	printf("FS size:\t\t\t%.2f Mbytes\n", cluster * bs.BS_SecPerClus * SwapInt16(bs.BS_BytesPerSec) / (float) (1024*1024));
+	printf("FS size:\t\t\t%.2f MiBytes\n", (float)cluster * bs.BS_SecPerClus * SwapInt16(bs.BS_BytesPerSec) / (1024.0*1024));
 	if (FATType == 32) {
 		if (getFATEntry(fd, &bs, SwapInt32(bs.FATxx.FAT32.BS_RootClus), &value) == -1) {
 			myerror("Failed to get FAT enry!");
 			return -1;
 		}
-		printf("FAT32 root directory first cluster: 0x%x, Data offset: 0x%x, FAT entry: 0x%x\n",
+		printf("FAT32 root directory first cluster: 0x%x, Data offset: 0x%llx, FAT entry: 0x%x\n",
 			SwapInt32(bs.FATxx.FAT32.BS_RootClus),
-			getClusterOffset(&bs, SwapInt32(bs.FATxx.FAT32.BS_RootClus)), value);
+			(unsigned long long)getClusterOffset(&bs, SwapInt32(bs.FATxx.FAT32.BS_RootClus)), value);
 	}
 
 	fclose(fd);
@@ -1479,6 +1480,7 @@ int main(int argc, char *argv[]) {
 	return 0;*/
 
 	/* Default (1) is normal order, use -1 for reverse order. */
+	
 	OPT_REVERSE = 1;
 	
 	opterr=0;
